#define SCROLL_SPEED 0.5

#include <time.h>
#include "std.h"
#include <math.h>

BITMAP *buffer = 0;
int gfx_mode=GFX_AUTODETECT, vid_width=640, vid_height=480, vid_bpp=16;

Player *pl_user = 0;
vector <Player *> players;
Vec2 camera_pos;
bool exit_game;
BITMAP *gui_buffer;
int game_counter;
int game_step;
BITMAP *terrain_buf;
BITMAP *logo =0;
int *terrain_colors;
vector<Robot*> robots;

bool victory;

struct BuildingInfo
{
	int dirs;
	int materials;
	char *img;
};


/*
#define B_HOME			0
#define B_WATER			1
#define B_LIFESUPPORT	2
#define B_FACTORY		3
#define B_CONNECTOR		4
#define B_GENERATOR		5

#define B_JELLYBASE		6
#define B_JELLYFACTORY	7
*/

BuildingInfo building_info[] =
{ 
	{ 15, 2000, "main_module" },
	{ DIR_E|DIR_W, 30, "watertank" },
	{ 15, 60, "lifesupport"},
	{ 15, 80, "factory" },
	{ 15, 5, "connector" },
	{ 15, 70, "powergen"},
	{ 15, 60, "j_base"},
	{ 15, 60, "j_factory"},
	{ 15, 60, "j_lifesupport"},
};

Selector current_sel;

void Player::Init (Vec2 rp)
{
	if (is_user)
	{
		root = new Building (this, rp, "main_module", B_HOME);
		materials = 500;
	}
	else
	{
		// cpu player
		root = new Building (this, rp, "j_base", B_JELLYBASE);
		materials = 1000;

		ai_task = TASK_STARTUP;
	}
	power = 1000;
	population = 500;
	fuel = 10;
	root->UpdateBox ();
}

void Player::Simulate ()
{
	if (is_user)
	{
		int numgen = root->CountType (B_GENERATOR);

		// all generators plus homebase
		int maxpower = (numgen+1) * 400;
		power += (numgen+0.5) * game_step/8;
		if(power > maxpower) power=maxpower;

		int numh2o = root->CountType (B_WATER);
		water_cap = numh2o * 50;

		int numlifesupport = root->CountType (B_LIFESUPPORT);
		int maxpop = numlifesupport * 2000;

		population += numlifesupport * game_step / 8;
		if (population > maxpop) population = maxpop;
		fuel += (numgen+0.5) *game_step/20;
	}
	else
	{
		int numls = root->CountType (B_JELLYLIFESUPPORT);
		int maxp = numls * 1000;

		population += numls * game_step / 4;
		
		int numfactory = root->CountType(B_JELLYFACTORY);
		int maxpower =(numfactory +1) * 400;

		power += (numfactory+0.5f) * game_step/8;
	}
}

bool AI_BuildAliens (Building *b, Player *pl)
{
	int a;
	if(b->type == B_JELLYFACTORY)
	{
		Vec2 p = b->world_pos;
		p.y -= 100;

		if (!Check_Circle_Collision (p, 20))
		{
			Robot *r = new rJellyAlien (pl);
			r->pos = p;
			return true;
		}
		else 
			return false;
	}

	for (a=0;a<b->childs.size ();a++)
		if (AI_BuildAliens (b->childs[a], pl))
			return true;

	return false;
}

bool AI_BuildBuilding (Player *pl, Building *b, int type)
{
	int a;

	BITMAP *bmp = GetImg (building_info[type].img);
	
	for (a=0;a<b->buildspots.size ();a++)
	{
		Building::BuildSpot *spot = b->buildspots [a];
		BBox box;
		box.min = spot->pos - Vec2 (bmp->w / 2, bmp->h/ 2);
		box.max = spot->pos + Vec2 (bmp->w / 2, bmp->h/ 2);

		if (Col_IsFree (&box))
		{
			Vec2 dir = GetVecDir (spot->dir);
			Vec2 pos = dir * Vec2 (b->game_img->w/2 + 10 + bmp->w/2,b->game_img->h/2 + 10 + bmp->h/2);

			Building *nb = new Building (pl, pos, building_info[type].img, type, b);
			b->UpdateBox ();
			return true;
		}
	}

	for (a=0;a<b->childs.size();a++)
		if (AI_BuildBuilding (pl, b->childs [a], type)) return true;

	return false;
}

void Player::DoAI ()
{
	ai_startup_stage += game_step;
	if (ai_startup_stage > 2000)
	{
		AI_BuildBuilding (this, root, B_JELLYFACTORY);

		int a;
		for (a=0;a<2;a++)
			AI_BuildAliens (root, this);
		ai_startup_stage = 0;
	}

/*	switch (ai_task)
	{
	case TASK_STARTUP:
		ai_startup_stage ++;
		if (ai_startup_stage < 3)
		{
			Require (0, 30, 100, 100, 0);
		}
		else if(ai_startup_stage >= 3)
		{
			Require (0, 30, 0, 60, 0);
			AI_BuildBuilding (this, root, B_JELLYLIFESUPPORT);

			ai_task = TASK_PRODUCEALIENS;
		}
		break;
	case TASK_LIFESUPPORT:
		if (population > 1000 * root->CountType(B_JELLYFACTORY) + 100)
		{
			ai_task = TASK_PRODUCEALIENS;
			break;
		}

		if (Require (0, 30, 0, 60, 0))
		{
			AI_BuildBuilding(this, root, B_JELLYLIFESUPPORT);
		}
		break;
	case TASK_PRODUCEALIENS:
		if (power < 500)
		{
			ai_task = TASK_PRODUCEFACTORY;
			break;
		}

		if (!AI_BuildAliens (root, this))
		{
			ai_task = TASK_PRODUCEFACTORY;
			break;
		}

		break;
	case TASK_PRODUCEFACTORY:
		if (population < 500)
		{
			ai_task = TASK_LIFESUPPORT;
			break;
		}

		if (Require (0, 0, 100, 50, 0))
			AI_BuildBuilding (this, root, B_JELLYFACTORY);

		break;
	}*/
}


bool Player::Require (int w,int p, int pop, int m, int f)
{
	if (water_cap < w || power < p || population < pop || materials < m || fuel < f)
		return false;

	power-=p;
	population-=pop;
	materials-=m;
	fuel-=f;

	return true;
}


Robot::Robot (Player *pl, const char *img_name, float s)
{
	robots.push_back (this);

	img = GetImg (img_name);
	ASSERT(img);
	owner=pl;angle=0;
	dir_length = 0.0f;
	speed=s;
	radius = 30;
	damage=0;
	destroy=false;
}

Robot::~Robot()
{ }

void Robot::Unreg()
{
	RemoveFromContainer (robots, this);

/*	if (array_index < robots.size() - 1)
	{
		swap (robots [array_index], robots.back ());
		robots [array_index]->array_index = array_index;
	}*/
}

void Robot::Move ()
{
	float s = 10 * game_step / 1000.0f;
	if(owner->power < s)
		return;
	owner->power -= s;

	dir_length -= game_step / 1000.0f;

	RotateTo (cur_angle, angle, 180 / 1000.0f * game_step);

	if(dir_length > 0)
	{
		dir = GetAngleDir (cur_angle);

		Vec2 dest = pos + (dir * (speed * game_step / 1000.0f));

		BBox box; // i'm not in the mood for oriented bbox'es
		box.min = dest - Vec2 (radius, radius);
		box.max = dest + Vec2 (radius, radius);

		Robot *abot;
		if (!Col_IsFree (&box))
		{
			if (fabs(cur_angle - angle) < 5.0f)
				ReOrient ();
			// else just wait for right rotation
		}
		else if(Check_Robot_Collision (this))
		{
			if (fabs(cur_angle - angle) < 5.0f)
				angle += 180.0f;

			pos = dest;
		}
		else
			pos = dest;
	}
	else
		ReOrient ();
}


void Robot::Draw (BBox &cambox)
{
	BBox t;
	t.min = pos - Vec2 (radius, radius);
	t.max = pos + Vec2 (radius, radius);

	if (!cambox.Hits (t))
		return;

	rotate_sprite (buffer, img, pos.x - cambox.min.x - img->w/2, 
		pos.y - cambox.min.y - img->h/2, ftofix (cur_angle / 360.0f * 256));

//	if (Check_Robot_Collision (this))
//		circlefill (buffer, pos.x - cambox.min.x, pos.y - cambox.min.y, radius, 0xffffffff);
		
//	Vec2 p = pos + dir * 30 - cambox.min;
//	line (buffer, pos.x - cambox.min.x, pos.y - cambox.min.y, p.x, p.y,0xffffffff);
}

rAttacker::rAttacker (Player *pl)
: Robot (pl, "arobot", 50.0f)
{
	laser_load = 100;
	shoot_bot=0;
}

void rAttacker::ReOrient ()
{
	angle = rand () % 360;
	dir_length = 3 + rand ()%5;
}

void rAttacker::TryAttack (Robot *a)
{
	if (!a->owner->is_user)
		shoot_bot = a;
}

void rAttacker::Move ()
{
	Robot::Move ();

	BBox box;

	laser_load +=game_step;
	if (laser_load > 500)
		laser_load=500;

	Robot *bot;
	bot = Find_Bot (pos, 100.0f);
	if (bot && laser_load==500)
	{
		laser_load=0;
		Add_Laser (bot, this);
	}

	Vec2 r(100,100);
	box.min = pos - r;
	box.max = pos + r;
	Building *b = Col_FindB (&box);

	if (b&&laser_load == 500)
	{
		if (!b->player->is_user)
		{
			laser_load = 0;
			Add_Laser (b, this);
		}
	}
}


void R_Destroy_Building (Building *bld)
{
	if (bld->parent)
	{
		RemoveFromContainer (bld->parent->childs, bld);
		SAFE_DELETE(bld);
	}
	else
	{
		int a;
		for(a=0;a<players.size();a++)
		{
			if (players[a]->root == bld)
			{
				victory = true;
				break;
			}
		}
	}
}

void R_Destroy_Robot(Robot *r)
{
	r->Unreg ();
	SAFE_DELETE(r);
}

void Destroy_Building(Building *b)
{
	b->destroy=true;
}

void Destroy_Robot(Robot *r)
{
	r->destroy=true;
}



rCollector::rCollector (Player *pl)
: Robot (pl,"crobot", 50.0f)
{
	materials =0;
}

void rCollector::Move ()
{
	Robot::Move ();

	owner->materials += 0.05f * game_step;
}

void rCollector::ReOrient ()
{
	angle = rand () % 360;
	dir_length = 1 + rand ()%4;
}

rJellyAlien::rJellyAlien(Player *pl)
	: Robot(pl, "j_robot", 40.0f)
{
	blub_load = 100;
}

void rJellyAlien::ReOrient ()
{
	angle = rand () % 360;
	dir_length = 2 + rand ()%4;
}

void rJellyAlien::Move ()
{
	Robot::Move ();

	if (rand () % 100 < 20)
		owner->materials += .1f * game_step;
}


Building::Building (Player *pl, Vec2 pos, const char *fn, int t, Building *p)
{
	player =pl;
	damage=0;
	destroy=false;
	game_img = GetImg (fn);
	ASSERT(game_img);

	relative = pos;
	if (p)
	{
		parent = p;
		p->AddChild (this);
		world_pos = pos + p->world_pos;
	}
	else
	{
		world_pos = pos;
		parent = 0;
	}

	Vec2 size = Vec2 (game_img->w, game_img->h) * 0.5f;
	box.min = world_pos - size;
	box.max = world_pos + size;

	type = t;

	if (player->is_user)
		sel = Add_BuildingSelector (this);
	else
		sel=0;

	int dirs = building_info [t].dirs;
	if (dirs & DIR_S)
		AddSpot (DIR_S, 0, -1);
	if (dirs & DIR_W)
		AddSpot (DIR_W, -1, 0);
	if (dirs & DIR_N)
		AddSpot (DIR_N, 0, 1);
	if (dirs & DIR_E)
		AddSpot (DIR_E, 1, 0);

	if (parent)
		parent->UpdateBox ();

	pl->materials -= building_info [type].materials;
}

int Building::CountType (int t)
{
	int a, c=0;

	if (type == t)
		c++;

	for(a=0;a<childs.size();a++)
		c += childs[a]->CountType(t);
	return c;
}

void Building::AddChild (Building *b)
{
	childs.push_back (b);
}



Building::BuildSpot::~BuildSpot()
{
	if (sel)
		Free_Selector (sel);
}

Building::~Building()
{
	if (sel)
		Free_Selector (sel);

	DeleteContainer (buildspots);
	DeleteContainer (childs);
}


void Building::AddSpot (int d, int x,int y)
{
	BuildSpot *s = new BuildSpot;

	Vec2 size = box.max - box.min;
	s->pos = world_pos + size * Vec2 (x,y);
	s->building = this;
	s->dir = d;

	if (player->is_user)
	{
		s->sel = Add_BuildSpotSelector (s);
		if (!Col_IsFree (&s->sel->box))
		{
			delete s;
			return;
		}
	}

	buildspots.push_back (s);
}


void Building::Draw (BBox *cambox)
{
	int a;

	if (!cambox->Hits (totalbox))
		return;

	if (player->is_user)
	{
		for (a=0;a<childs.size();a++)
		{
			Vec2 cp = childs[a]->world_pos;
			int s = 5;
			rectfill (buffer, cp.x - s - cambox->min.x, cp.y - s - cambox->min.y, world_pos.x + s - cambox->min.x, world_pos.y + 10 - cambox->min.y, makecol (157, 145, 183));
			s = 6;
			rect (buffer, cp.x - s - cambox->min.x, cp.y - s - cambox->min.y, world_pos.x + s - cambox->min.x, world_pos.y + 10 - cambox->min.y, makecol (80, 80, 80));
		}
	}

	for (a=0;a<childs.size ();a++)
		childs[a]->Draw (cambox);

	if (!player->is_user)
	{
		drawing_mode (DRAW_MODE_TRANS, 0 ,0,0);
		set_trans_blender (0,0,0,80);
		draw_trans_sprite (buffer, game_img, world_pos.x - cambox->min.x - game_img->w/2, world_pos.y - cambox->min.y - game_img->h/2);
		solid_mode ();
	}
	else
		draw_sprite (buffer, game_img, world_pos.x - cambox->min.x - game_img->w/2, world_pos.y - cambox->min.y - game_img->h/2);
}


void Building::UpdateBox ()
{
	totalbox = box;

	int a;
	for(a=0;a<childs.size();a++)
	{
		childs[a]->UpdateBox ();
		totalbox.Extend (childs[a]->totalbox);
	}
}


void Draw_Game_Stats (Player *pl, int x, int y, int col)
{
	textprintf (gui_buffer, font, x, y, col, "Population: %d", pl->GetPopulation());
	textprintf (gui_buffer, font, x, y + 15, col, "H2O Capacity: %d", pl->WaterCap ());
	textprintf (gui_buffer, font, x, y + 30, col, "Power: %d", (int)pl->power);
	textprintf (gui_buffer, font, x, y + 45, col, "CH4: %d", pl->fuel);
	textprintf (gui_buffer, font, x, y + 60, col, "Construction");
	textprintf (gui_buffer, font, x, y + 70, col, "Materials: %f", pl->materials);
}

#define TRI_STEP 8

inline void terrain_triangle (BITMAP *bmp, int x1,int y1, int x2,int y2,int x3,int y3,int col)
{
	int cx=(int)(camera_pos.x)%TRI_STEP, cy=(int)(camera_pos.y)%TRI_STEP;
	triangle (bmp,x1-cx,y1-cy,x2-cx,y2-cy,x3-cx,y3-cy,col);
}


void Draw_Background ()
{
	int w = buffer->w / TRI_STEP + 1;
	int h = buffer->h / TRI_STEP;
	int x,y;

	int sx = camera_pos.x,sy=camera_pos.y;
	int ex = sx + buffer->w,ey =sy+buffer->h;

	sx /= TRI_STEP;
	sy /= TRI_STEP;
	ex /= TRI_STEP;
	ey /= TRI_STEP;

	if (sx < 0) sx=0;
	if (sy < 0) sy=0;
	if (ex >= LEVEL_W) ex = LEVEL_W-1;
	if (ey >= LEVEL_H) ey = LEVEL_H-1;

	int ox = (int)(camera_pos.x) % TRI_STEP,
		oy = (int)(camera_pos.y) % TRI_STEP;

	for(x=sx;x<=ex;x++)
		for(y=sy;y<=ey;y++)
		{
			int col = terrain_colors [y * LEVEL_W + x];
			int tx = x - sx;
			int ty = y - sy;

			triangle (buffer, tx*TRI_STEP-ox, ty*TRI_STEP-oy, tx*TRI_STEP+TRI_STEP/2-ox, (ty+1)*TRI_STEP-oy,tx*TRI_STEP-TRI_STEP/2-ox, (ty+1)*TRI_STEP-oy, col);
			triangle (buffer, tx*TRI_STEP-ox, ty*TRI_STEP-oy, tx*TRI_STEP+TRI_STEP-ox, ty*TRI_STEP-oy, tx*TRI_STEP+TRI_STEP/2-ox, (ty+1)*TRI_STEP-oy, col);
		}
#undef TRI_STEP
}

void Generate_Terrain_Colors ()
{
	terrain_colors = new int [LEVEL_W * LEVEL_H];

	int x,y;
	for (x=0;x<LEVEL_W;x++)
		for (y=0;y<LEVEL_H;y++)
		{
			terrain_colors [y*LEVEL_W+x] = makecol (150 + rand () % 45, 40 + rand () % 30, rand () % 30);
		}
}

void Draw_Game()
{
	BBox cambox;
	int a;

	cambox.min = camera_pos;
	cambox.max = camera_pos + Vec2 (vid_width - GAME_UI_W, vid_height);

	Draw_Background ();

	for (a=0;a<players.size();a++)
	{
		players[a]->root->Draw (&cambox);
	}

	for (a=0;a<robots.size();a++)
	{
		robots[a]->Draw (cambox);
	}

	Draw_Lasers (cambox);

//	Draw_Circles (cambox);

	Draw_Selector (&current_sel, makecol (200, 200, 0), &cambox);

	float dis;
	Selector *sel;
	sel = Find_Nearest_Selector (cambox.min + Vec2 (mouse_x, mouse_y), &dis);
	if (dis < 50)
		Draw_Selector (sel, makecol (160,160,160), &cambox);

#ifdef _DEBUG
	textout (buffer, font, "Game Running", 0, 0, 0xffffffff);
	textprintf (buffer, font, 0, 20, 0xffffffff, "Camera: %d %d", (int)camera_pos.x, (int)camera_pos.y);
#endif
	draw_sprite (buffer, mouse_sprite, mouse_x, mouse_y);
}


void Check_DestructB(Building *b)
{
	if(b->destroy)
		R_Destroy_Building (b);
	else
	{
		int a;
		for(a=0;a<b->childs.size();a++)
			Check_DestructB (b->childs[a]);
	}
}

void Handle_Game ()
{
	int a;
	float s = SCROLL_SPEED * game_step;
	bool gen=false;
	if (mouse_x == 0 || key[KEY_LEFT])
	{ camera_pos.x -= s; gen=true; }
	if (mouse_y == 0 || key[KEY_UP])
	{ camera_pos.y -= s; gen=true; }
	if (mouse_x == vid_width-1 || key[KEY_RIGHT])
	{ camera_pos.x += s; gen=true; }
	if (mouse_y == vid_height-1 || key[KEY_DOWN])
	{ camera_pos.y += s; gen=true; }

	if (camera_pos.x < 0)
		camera_pos.x = 0;
	if (camera_pos.y < 0)
		camera_pos.y = 0;
	if (camera_pos.x > LEVEL_W - buffer->w)
		camera_pos.x = LEVEL_W - buffer->w;
	if (camera_pos.y > LEVEL_H - buffer->h)
		camera_pos.y = LEVEL_H - buffer->h;

	static int prev_mb =0;
	int mb = mouse_b;

	if ((mb & 1) && !(prev_mb & 1))
	{
		float dis;
		Selector *s = Find_Nearest_Selector (Vec2 (mouse_x,mouse_y)+camera_pos, &dis);

		if (s) {
			if(dis < 50)
				current_sel = *s;
		}
	}

	prev_mb = mb;

	Fill_CollisionBuf ();
	for (a=0;a<robots.size();a++)
		robots[a]->Move ();

	for (a=0;a<players.size();a++)
	{
		if (!players [a]->is_user)
			players [a]->DoAI ();

		players [a]->Simulate ();
	}

	Handle_Lasers ();

	for (a=0;a<players.size();a++)
	{
		Check_DestructB(players[a]->root);
	}

	while(1)
	{
		for (a=0;a<robots.size();a++)
		{
			if (robots[a]->destroy)
			{
				Robot *b=robots[a];
				R_Destroy_Robot(b);
				continue;
			}
		}
		break;
	}

	if (victory)
	{
		alert ("You have defeated the evil jelly aliens", "Mars is all yours now!", 0,"OK",0,0,0);
	}
}

bool Init_Game ()
{
	if (!Load_GameData ())
	{
		al_trace ("Failed to load game data\n");
		return false;
	}

	pl_user = new Player;
	pl_user->is_user = true;
	players.push_back (pl_user);

	pl_user->Init (Vec2 (300, 300) );

	current_sel = *pl_user->root->sel;
	Generate_Terrain_Colors ();

	/* create enemies */

	Player *pl = new Player;
	players.push_back (pl);
	pl->is_user = false;
	pl->Init (Vec2 (LEVEL_W - 400, 500));

	logo = GetImg ("logo");

	Gen_CollisionBuf ();
	victory=false;

	return true;
}

void Free_Game ()
{
	DeleteContainer (robots);
	DeleteContainer (players);
	players.clear();
	robots.clear ();
	Free_GameData ();

	SAFE_DELETE_ARRAY(terrain_colors);
	logo=0;
}


void Check_Buttons(TList<Button> *bl)
{
	static int prev_b = 0;
	int mx=mouse_x-vid_width+GAME_UI_W, my=mouse_y, mb=mouse_b;
	for (TList<Button> *b = bl; b; b = b->next)
	{
		if (b->is_label)
			continue;

		b->fade_timer -= game_step;
		if(b->fade_timer < 0) b->fade_timer = 0;

		if (b->Check (mx, my) && !(prev_b & 1) && (mb & 1))
		{
			b->proc ();
			b->fade_timer = 500;
		}
	}

	prev_b = mb;
}


void Create_Building (int t)
{
	if (current_sel.b)
	{
		alert ("You can't build on a building", 0, 0, "ok", 0, 0,0);
		return;
	}

	Building::BuildSpot *s = current_sel.s;
	ASSERT(s);

	if (!(building_info [t].dirs & s->dir))
	{
		alert ("This type of building does not support this direction", 0, 0, "ok", 0,0,0);
		return;
	}

	if (pl_user->materials < building_info [t].materials)
	{
		alert ("You don't have enough building materials for this.", "You will need collector robots to get more materials", 0, "OK", 0,0,0);
		return;
	}

	BITMAP *img = GetImg (building_info [t].img);
	Vec2 dir = GetVecDir (s->dir);
	Vec2 pos = dir * Vec2 (img->w/2 + 10 + s->building->game_img->w/2,img->h/2 + 10 + s->building->game_img->h/2);

	Building *nb=0;
	nb = new Building (pl_user, pos, building_info [t].img, t, s->building);

	RemoveFromContainer (s->building->buildspots, s);
	delete s;

	if (nb)
		current_sel = *nb->sel;
}

static void PlaceWaterStorage ()
{
	Create_Building (B_WATER);
}

static void PlaceFactory ()
{
	Create_Building (B_FACTORY);
}


static void PlaceLifeSupport ()
{
	Create_Building (B_LIFESUPPORT);
}

static void PlaceConnector()
{
	Create_Building (B_CONNECTOR);
}

static void PlaceGenerator()
{
	Create_Building (B_GENERATOR);
}

void Add_Robot (Robot *r)
{
	ASSERT(current_sel.b->type == B_FACTORY);

	r->pos = current_sel.b->world_pos + Vec2 (0, 100);
}

static void Create_Attacker_Robot()
{
	if (pl_user->Require (0, 40, 0, 30, 3))
		Add_Robot (new rAttacker (pl_user));
	else
		alert ("Not enough resources",0,0,"OK",0,0,0);
}

static void Create_Collector_Robot ()
{
	if(pl_user->Require (0, 30, 0, 10, 2))
		Add_Robot (new rCollector (pl_user));
	else
		alert ("Not enough resources",0,0,"OK",0,0,0);
}

static void ExitProc() { exit_game = true; }

TList<Button>* AddButton (TList<Button> *& bl, const char *text, void (*proc)())
{
	int bs;
	if (!bl)
		bs = vid_height - 300;
	else
		bs = bl->y + 30;

	TList<Button> *b = new TList<Button>;
	b->text = text;
	b->proc = proc;
	b->Place (10, bs, GAME_UI_W - 20, 20);
	b->Link (bl);

	if (!proc)
		b->is_label = true;

	return b;
}

void Create_Exit_Button(TList<Button> *&blist)
{
	TList<Button>* b = new TList<Button>;
	b->text = "Exit Game";
	b->proc = ExitProc;
	b->Place (10, vid_height-40, GAME_UI_W - 20, 20);
	b->Link (blist);
}

TList<Button> *Create_Buttons ()
{
	TList<Button> *blist = 0;

	AddButton(blist, "Building Menu:",0);
	AddButton(blist, "Water Storage", PlaceWaterStorage);
	AddButton(blist, "Robot Factory", PlaceFactory);
	AddButton(blist, "Life Support", PlaceLifeSupport);
	AddButton(blist, "Connector", PlaceConnector);
	AddButton(blist, "Power Generator", PlaceGenerator);

	Create_Exit_Button (blist);

	return blist;
}

TList<Button> *Robot_Buttons()
{
	TList<Button> *blist = 0;

	AddButton (blist, "Robot Menu:", 0);
	AddButton (blist, "Attacker Robot", Create_Attacker_Robot);
	AddButton (blist, "Collector Robot", Create_Collector_Robot);

	Create_Exit_Button(blist);

	return blist;
}



void Draw_Buttons (TList<Button> *list)
{
	text_mode(-1);
	for (TList<Button> *b = list;b;b=b->next)
	{
		int  c = makecol (255, MIN(b->fade_timer,255), 0);
		if (!b->is_label)
		{
			rect (gui_buffer, b->x, b->y, b->w+b->x, b->h+b->y, b->Check (mouse_x-vid_width+GAME_UI_W, mouse_y) ? makecol (100, 100, 0) : makecol(255,0,0));
			rect (gui_buffer, b->x+1, b->y+1, b->w+b->x-1, b->h+b->y-1,c);
		}

		textout_centre (gui_buffer, font, b->text, b->x + b->w / 2, b->y + b->h/2 - 4, ~c);
	}
}

void Draw_UI(TList<Button> *buttons)
{
	int dlg_fg = makecol (200,60,60);
	int dlg_bg = makecol (60, 50, 50);

	clear_to_color (gui_buffer, dlg_bg);
	rect (gui_buffer, 0, 0, gui_buffer->w -1, gui_buffer->h-1, dlg_fg);
	if(buttons) Draw_Buttons (buttons);
	Player *pl=pl_user;
	if (key[KEY_F3])
		pl = players[1];

	Draw_Game_Stats (pl, 10, 70, makecol (0,0,255));

	draw_sprite(gui_buffer, logo, 0, 0);
	draw_sprite(gui_buffer, mouse_sprite,mouse_x-vid_width+GAME_UI_W,mouse_y);
}


void Game_Timer_Proc()
{
	game_counter ++;
}
END_OF_FUNCTION(Game_Timer_Proc);

void Game_Loop ()
{
	col_objs.reserve (200);

	if (!Init_Game ())
	{
		alert ("Failed to initialize game",0, 0, "OK", 0, 0, 0);
		return;
	}

	buffer = create_bitmap (vid_width-GAME_UI_W, vid_height);
	terrain_buf = create_bitmap (vid_width-GAME_UI_W, vid_height);
	gui_buffer = create_bitmap (GAME_UI_W, vid_height);

	TList<Button> *create_buttons = Create_Buttons();
	TList<Button> *robot_buttons = Robot_Buttons ();
	TList<Button> *base_menu = 0;
	Create_Exit_Button (base_menu);

	game_counter=0;
	LOCK_VARIABLE(game_counter);
	LOCK_FUNCTION(Game_Timer_Proc);
	install_int (Game_Timer_Proc, 20);

	exit_game=false;
	while (!key [KEY_ESC] && !exit_game)
	{
		game_step = game_counter * 20;
		game_counter = 0;

		TList<Button> *cur_buttons = 0;
		if(current_sel.b) {
			if (current_sel.b->type == B_FACTORY)
				cur_buttons = robot_buttons;
			else
				cur_buttons = base_menu;
		}
		else
			cur_buttons = create_buttons;

		if(cur_buttons)
			Check_Buttons (cur_buttons);
		Handle_Game ();
		if (victory)
			break;

		Draw_Game ();
		Draw_UI (cur_buttons);
		blit (buffer, screen, 0,0,0,0, buffer->w, buffer->h);
		blit (gui_buffer, screen, 0,0, vid_width - GAME_UI_W, 0, GAME_UI_W, vid_height);

	}

	remove_int (Game_Timer_Proc);

	robot_buttons->RemoveList ();
	create_buttons->RemoveList ();
	base_menu->RemoveList();

	destroy_bitmap (buffer);
	destroy_bitmap (terrain_buf);
	destroy_bitmap (gui_buffer);
	Free_Game ();
	Free_CollisionMem ();
}

void Exit_Game () { menu_close=true; }

int Do_Main_Menu ()
{
	MenuItem mainmenu[] = {
		{ Game_Loop, "Start Game" },
		{ Exit_Game, "Exit Game" }, 
		{ 0 }
	};

	Show_Menu (mainmenu, "data/menu_back.bmp");
	return 0;
}

int Assert_Handler (const char *msg)
{
	allegro_message (msg);
	exit (-1);
	return 0;
}

int main (int argc, char *argv[])
{
	bool windowed=false;
	allegro_init ();

	srand (time (0));

	install_keyboard ();
	install_mouse ();
	install_timer ();

	register_assert_handler (Assert_Handler);

	if (argc > 1)
	{
		for (int a=1;a<argc;a++) {
			if (!ustricmp (argv [a], "-bpp") && a+1 < argc)
			{
				vid_bpp = atoi (argv[++a]);
				if (vid_bpp < 15)
				{
					al_trace ("This program needs at least 15 bits per pixel.\n");
					return -1;
				}
			}
			if(!ustricmp (argv[a], "-window"))
			{
				windowed=true;
			}
		}
	}

	set_color_depth (vid_bpp);
	if (set_gfx_mode (windowed ? GFX_AUTODETECT_WINDOWED : GFX_AUTODETECT_FULLSCREEN, vid_width, vid_height, 0,0) < 0)
	{
		al_trace ("Failed to setup graphics mode (%dx%dx%d). Error: %s\n", vid_width, vid_height, vid_bpp, allegro_error);
		return -1;
	}

	gui_fg_color = makecol (250,90,90);
	gui_bg_color = makecol (50, 50, 50);

	int ret = Do_Main_Menu ();

	allegro_exit ();

	return ret;
}
END_OF_MAIN();

