
#ifndef STD_H
#define STD_H

#ifdef _DEBUG
#define DEBUGMODE
#endif

#include <vector>
#include <algorithm>
using namespace std;

#define SAFE_DELETE(c) if(c) { delete (c); (c)=0; }
#define SAFE_DELETE_ARRAY(c) if(c) { delete[] (c); (c)=0; }


typedef unsigned char uchar;
typedef unsigned short ushort;
typedef unsigned long ulong;
typedef unsigned int uint;

// i had these as templates but gcc didn't like them
#define DeleteContainer(_C) { for(int a=0;a<_C.size();a++) if (_C[a]) delete _C[a]; _C.clear(); }
#define RemoveFromContainer(_C, _Obj)  {					\
	for(int a=0;a<_C.size();a++) if(_C[a] == (_Obj))		\
	{ if(a<_C.size()-1) swap(_C[a], _C.back()); _C.pop_back(); break; } }

#include <allegro.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "linklist.h"


#define GAME_UI_W 160
#define LEVEL_W 1500
#define LEVEL_H 1000


// simple 2d vector with a few operators
struct Vec2
{
	float x, y;

	Vec2 (float X=0.0f, float Y=0.0f) {x=X;y=Y; }
	Vec2 operator+(const Vec2& a) { return Vec2(x+a.x, y+a.y); }
	Vec2 operator-(const Vec2& a) { return Vec2(x-a.x, y-a.y); }
	Vec2 operator*(float a) { return Vec2(x*a,y*a); }
	Vec2 operator*(Vec2 a) { return Vec2(x*a.x,y*a.y); }
	Vec2& operator+=(const Vec2& a) { x+=a.x; y+=a.y; return *this; }
	Vec2& operator-=(const Vec2& a) { x-=a.x; y-=a.y; return *this; }
};

struct BBox
{
	Vec2 min, max;

	void Extend (const BBox &b);
	bool Hits (const BBox &b);
	void Output ();
};

struct ColObj;
struct Building;

void Col_Add (ColObj *obj);
void Col_Remove (ColObj *obj);
bool Col_IsFree (BBox *location);
Building* Col_FindB (BBox *l);


// collision obj
struct ColObj
{
	ColObj () { Col_Add (this); }
	~ColObj () { Col_Remove (this); }

	BBox box;
};


#define B_HOME			0
#define B_WATER			1
#define B_LIFESUPPORT	2
#define B_FACTORY		3
#define B_CONNECTOR		4
#define B_GENERATOR		5

#define B_JELLYBASE		6
#define B_JELLYFACTORY	7
#define B_JELLYLIFESUPPORT	8

#define DIR_N	1
#define DIR_E	2
#define DIR_S	4
#define DIR_W	8

struct Selector;
struct Player;

#define TASK_STARTUP			0
#define TASK_PRODUCEALIENS		1
#define TASK_PRODUCEFACTORY		2
#define TASK_LIFESUPPORT		3

struct Building : ColObj
{
	Building (Player *pl, Vec2 pos, const char *fn, int type, Building *p = 0);
	~Building ();

	struct BuildSpot
	{
		BuildSpot () { b=0; dir=0; sel=0; building=0; }
		~BuildSpot();

		Building *building;
		Vec2 pos; //relative
		Selector *sel;
		int b;
		int dir;
	};

	int CountType (int t);
	void Draw (BBox *cambox);
	void UpdateBox ();
	void AddSpot (int d, int x, int y);
	void AddChild (Building *b);

	Vec2 relative, world_pos;
	Building *parent;
	BITMAP *game_img;
	BBox totalbox;
	Selector *sel;
	int type;
	Player *player;
	int damage;
	bool destroy;

	vector <BuildSpot*> buildspots;
	vector <Building*> childs;
};



struct Robot
{
	Robot (Player *pl ,const char *img_name, float s);
	~Robot ();

	void Unreg ();

	void Draw (BBox &cambox);
	virtual void Move ();
	virtual void ReOrient () = 0;
	virtual void TryAttack(Robot *a){}
	
	float speed;
	Vec2 dir;
	float dir_length;
	Player *owner;
	float angle, cur_angle;
	Vec2 pos;
	BITMAP *img;
	int array_index;
	float radius;
	int damage;
	bool destroy;

	int ncbuf;
#define MAX_BOTCBUF		4
	int cbufs[MAX_BOTCBUF];
};

struct rCollector : Robot
{
	rCollector (Player *pl);
	void Move ();
	void ReOrient ();

	int materials;
};

struct rAttacker : Robot
{
	rAttacker (Player *pl);
	void Move ();
	void ReOrient ();
	void TryAttack (Robot *a);

	Robot *shoot_bot;
	int laser_load;
};

struct rJellyAlien : Robot
{
	rJellyAlien(Player *pl);

	void Move ();
	void ReOrient ();

	int blub_load;
};

struct Player
{
	Player () 
	{	root=0; is_user=false;
		water_cap=0;
		power=0;
		population=0;
		fuel=0;
		materials=0;
		ai_task=0;
		ai_robot_build=0;
		ai_startup_stage=0;
	}
	~Player () { SAFE_DELETE (root); }

	void Init (Vec2 pos);
	void Simulate ();
	void DoAI ();

	int GetPopulation () { return population / 100 + 10; }
	int WaterCap () { return water_cap + 25; }

	bool Require (int w, int power, int pop, int mat, int fuel);

	Building *root;
	bool is_user;

	int ai_task;
	int ai_robot_build;
	int ai_startup_stage;
	int water_cap;
	float power;
	int population;
	float materials;
	int fuel;
};

extern Player *pl_user;


struct MenuItem
{
	void (*proc)();
	char *text;
};


struct Button
{
	const char *text;
	void (*proc)();
	int x,y,w,h;
	int fade_timer;
	bool is_label;

	Button(){ text=0; x=y=w=h=0; proc=0;fade_timer=0;is_label=false; }
	void Place (int X, int Y, int W, int H) { x=X;y=Y;w=W;h=H; }
	bool Check (int X,int Y) { return X>=x && Y>=y && X<x+w && Y<y+h; }
};



struct Selector
{
	Selector () {b=0;s=0; }

	Building *b;
	Building::BuildSpot *s;

	Vec2 mid;
	BBox box;
};

void Destroy_Building (Building *bld);
void Destroy_Robot(Robot *r);

void Add_Laser(Building *b, Robot *bot);
void Add_Laser (Robot *abot, Robot *bot);
void Draw_Lasers(BBox &cambox);
void Handle_Lasers ();

void Show_Menu (MenuItem *m, const char *backg);
void Next_Frame ();
bool Load_GameData ();
void Free_GameData ();
SAMPLE *GetSound (const char *file);
BITMAP *GetImg (const char *file);

Selector* Add_BuildingSelector (Building *b);
Selector* Add_BuildSpotSelector (Building::BuildSpot *s);
void Free_Selector (Selector *s);
void Draw_Selector (Selector *s, int col, BBox *cambox);
Selector* Find_Nearest_Selector (Vec2 p, float *dis=0);
Robot *Find_Bot  (Vec2 pos, float r);

void Free_CollisionMem ();
void Gen_CollisionBuf ();
Robot* Check_Robot_Collision (Robot*b);
bool Check_Circle_Collision (Vec2 pos, float r);
void Fill_CollisionBuf ();
void Draw_Circles (BBox &cambox);

void RotateTo (float &ang, float d, float s);
void FixAngle (float &ang);
Vec2 GetAngleDir (float angle);
Vec2 GetVecDir (int dir);

extern bool menu_close;
extern BITMAP *buffer;
extern BITMAP *gui_buffer;
extern int vid_width, vid_height;
extern vector<ColObj*> col_objs;
extern vector<Robot*> robots;
extern int game_step;

#endif

