
#include "std.h"

vector <Selector*> selectors;
vector<ColObj*> col_objs;


bool menu_close = false;
static BITMAP *scr_buffer = 0;

#define MENU_H 30
#define MENU_SEL_COL makecol(255,255,0)
#define MENU_TEXT_COL makecol(125,125,125)

void Draw_Item (MenuItem *item, bool sel, int y)
{
	int w = text_length(font, item->text);
	int x = vid_width / 2 - (w / 2), c;

	if(sel)
		c = MENU_SEL_COL;
	else
		c = MENU_TEXT_COL;

	textout (scr_buffer, font, item->text, x, y, c);
}

void Draw_Menu (MenuItem *m, int sel)
{
	int a;
	int count;

	for(count=0;m[count].proc;count++);
	
	int y = vid_height / 2 - (count / 2) * MENU_H;

	text_mode (-1);

	for(a=0;m[a].proc;a++)
	{
		Draw_Item (&m[a], a == sel, y);
		y += MENU_H;
	}
}


void Show_Menu (MenuItem *m, const char *backg)
{
	int item;
	int count = 0;

	BITMAP *background;
	PALETTE pal;

	scr_buffer = create_bitmap (vid_width, vid_height);
	clear (scr_buffer);

	if(backg)
		background = load_bitmap (backg, pal);
	else
		background = 0;

	for(count = 0; m[count].proc; count ++)
		;

	menu_close = false;
	item = 0;
	while (!key[KEY_ESC] && !menu_close)
	{
		if(key[KEY_DOWN] || key[KEY_UP])
		{
			if(key[KEY_DOWN])
				item ++;
			if(key[KEY_UP])
				item --;
			if(item < 0) item = count-1;
			if(item >= count) item = 0;

			if(background)
				blit (background, scr_buffer, 0, 0, 0,0, background->w,background->h);
			else
				clear (scr_buffer);

			Draw_Menu (m, item);
			blit (scr_buffer, screen, 0,0,0,0,vid_width, vid_height);
			while (key[KEY_DOWN] || key[KEY_UP]);
		}

		if(key[KEY_ENTER] || key[KEY_SPACE])
		{
			menu_close = false;
			while (key[KEY_ENTER] || key[KEY_SPACE]);
			clear_keybuf ();
			m [item].proc ();

			while (key[KEY_ENTER] || key[KEY_SPACE] || key[KEY_ESC]);
			clear_keybuf ();

			if(menu_close)
				break;
		}

		if(background) blit (background, scr_buffer, 0, 0, 0,0, background->w,background->h);
		else clear (scr_buffer);

		Draw_Menu (m, item);
		blit (scr_buffer, screen, 0,0,0,0,vid_width, vid_height);
	}

	if(background)
		destroy_bitmap (background);

	destroy_bitmap (scr_buffer);
}


void BBox::Output ()
{
	TRACE ("BBox Min: %d %d  Max: %d %d\n",(int)min.x, (int)min.y, (int)max.x, (int)max.y);
}



void BBox::Extend (const BBox &b)
{
	if (min.x > b.min.x) min.x = b.min.x;
	if (min.y > b.min.y) min.y = b.min.y;
	if (max.x < b.max.x) max.x = b.max.x;
	if (max.y < b.max.y) max.y = b.max.y;
}


bool BBox::Hits (const BBox & b)
{
	return min.x <= b.max.x && max.x >= b.min.x &&
		min.y <= b.max.y && max.y >= b.min.y;
}



Selector* Add_BuildingSelector (Building *b)
{
	Selector *s = new Selector;
	s->b = b;
	s->box = b->box;
	s->mid = (s->box.min + s->box.max) * 0.5f;

	b->box.Output();

	selectors.push_back (s);
	return s;
}

Selector* Add_BuildSpotSelector (Building::BuildSpot *spot)
{
	Selector *s = new Selector;

	s->b = 0;
	s->s = spot;
	Vec2 d (20,20);
	s->box.min = spot->pos - d;
	s->box.max = spot->pos + d;
	s->mid = spot->pos;
	s->box.Output ();

	selectors.push_back (s);
	return s;
}


void Free_Selector (Selector *s)
{
	RemoveFromContainer (selectors, s);
	delete s;
}


void Draw_Selector (Selector *s, int col, BBox *cambox)
{
	rect (buffer, s->box.min.x - cambox->min.x, s->box.min.y - cambox->min.y, 
		s->box.max.x - cambox->min.x, s->box.max.y - cambox->min.y, col);

	//rect (buffer, s->mid.x - cambox->min.x - 10, s->mid.y - cambox->min.y - 10,
	//	s->mid.x - cambox->min.x + 10, s->mid.y - cambox->min.y + 10, col);
}


Selector* Find_Nearest_Selector (Vec2 p, float *store_dis)
{
	Selector *best = 0;
	float dis2 =0.0f;

	int a;
	for(a=0;a<selectors.size();a++)
	{
		Vec2 dif = selectors[a]->mid - p;
		float d2 = dif.x*dif.x + dif.y*dif.y;

		if (!best)
		{
			best = selectors [a];
			dis2 = d2;
			continue;
		}

		if (d2 < dis2)
		{
			dis2 = d2;
			best = selectors [a];
		}
	}

	if(store_dis)
		*store_dis = sqrt (dis2);

	return best;
}



void Col_Add (ColObj *obj)
{
	col_objs.push_back (obj);
}

void Col_Remove (ColObj *obj)
{
	RemoveFromContainer (col_objs, obj);
}

bool Col_IsFree (BBox *loc)
{
	int a;
	
	for (a=0;a<col_objs.size();a++)
	{
		if (loc->Hits (col_objs[a]->box))
			return false;
	}

	if (loc->min.x < 0 || loc->min.y < 0 || loc->max.x > LEVEL_W-1 || loc->max.y > LEVEL_H - 1)
		return false;

	return true;
}


Building* Col_FindB (BBox *loc)
{
	int a;
	
	for (a=0;a<col_objs.size();a++)
	{
		if (loc->Hits (col_objs[a]->box))
			return (Building *)col_objs[a];
	}

	return 0;
}



Vec2 GetVecDir (int dir)
{
	switch (dir)
	{
	case DIR_S:
		return Vec2 (0,-1);
	case DIR_W:
		return Vec2 (-1,0);
	case DIR_E:
		return Vec2 (1,0);
	case DIR_N:
		return Vec2 (0,1);
	}
	return Vec2 (0,0);
}



void FixAngle (float &ang)
{
	while (ang > 360)
		ang -= 360;
	while (ang < 0)
		ang += 360;
}



Vec2 GetAngleDir (float angle)
{
	Vec2 d;

	angle = 3.141593f * angle / 180;
	d.y = cos (angle);
	d.x = -sin (angle);
	return d;
}

//rotate d to ang with speed s
void RotateTo (float &ang, float d, float s)
{
	FixAngle (ang);
	FixAngle (d);

	float dif = fabs(ang-d);

	if(dif<180) {
		if(dif<=s)
			ang = d;
		else {
			if(ang > d) ang -= s;
			else ang += s;
		}
	} else {
		if(ang > d) ang+=s;
		else ang -=s;
		FixAngle (ang);
	}
}

#define CIRCLE_STEP 200

// This is a grid of circles, not rectangles !! :-)
struct CircleGrid
{
	CircleGrid () { bots=0;max=0; }
	~CircleGrid () { SAFE_DELETE_ARRAY(bots); }

	void Allocate (int n)
	{
		max = n;
		bots = new Robot* [n];
		num = 0;
	}
	void Add (Robot *bot)
	{
		bots [num ++] = bot;
	}

	int num;
	Vec2 pos;
	Robot **bots;
	int max;
};

const int cbuf_w = LEVEL_W / CIRCLE_STEP;
const int cbuf_h = LEVEL_H / CIRCLE_STEP;
float cbuf_r;
CircleGrid *circles;

void Gen_CollisionBuf()
{
	int x,y;

	cbuf_r = sqrt ((CIRCLE_STEP*CIRCLE_STEP)/2) + 10;
	circles = new CircleGrid [cbuf_w * cbuf_h];
	for (y = 0; y < cbuf_h; y++)
		for (x=0;x<cbuf_w;x++)
		{
			CircleGrid *g = &circles [y*cbuf_w + x];
			g->Allocate(50);
			g->pos = Vec2 ((x + 0.5f) * CIRCLE_STEP, (y + 0.5f) * CIRCLE_STEP);
		}
}


void Fill_CollisionBuf()
{
	int a, b;

	for (a=0;a<cbuf_h;a++)
	{
		for (b=0;b<cbuf_w;b++)
			circles [a*cbuf_w+b].num = 0;
	}

	for (a=0;a<robots.size();a++)
	{
		Robot *bot=robots[a];
		int n = cbuf_w * cbuf_h;
		Vec2 dif;

		bot->ncbuf = 0;

		for (b=0;b<n;b++)
		{
			dif = circles[b].pos - bot->pos;
			if (dif.x*dif.x + dif.y*dif.y < cbuf_r*cbuf_r )
			{
				circles [b].Add (bot);
				bot->cbufs [bot->ncbuf++] = b;

				if (bot->ncbuf >= MAX_BOTCBUF)
					break;
			}
		}
	}
}


void Draw_Circles (BBox &cambox)
{
	int a;
	for(a=0;a<cbuf_w*cbuf_h;a++)
		circle (buffer, circles[a].pos.x - cambox.min.x, circles[a].pos.y - cambox.min.y, cbuf_r, 0xffffffff);
}

Robot* Check_Robot_Collision (Robot *bot)
{
	int a,b;

	for (a=0;a<bot->ncbuf;a++)
	{
		CircleGrid *g = &circles [bot->cbufs [a]];
		Vec2 dif;
		float dis;

		for (b=0;b<g->num;b++)
		{
			Robot *cb = g->bots [b];

			if (cb == bot)
				continue;

			dif = bot->pos - cb->pos;
			dis = dif.x*dif.x+dif.y*dif.y;

			float r = cb->radius + bot->radius;
			if (dis < r*r)
			{
				return cb;
			}
		}
	}

	return 0;
}


bool Check_Circle_Collision (Vec2 pos, float radius)
{
	int a,b;

	for (a=0;a<cbuf_w*cbuf_h;a++)
	{
		CircleGrid *g = &circles [a];
		Vec2 dif;
		float dis;

		dif = g->pos - pos;
		if (dif.x*dif.x+dif.y*dif.y < cbuf_r *cbuf_r)
		{
			for (b=0;b<g->num;b++)
			{
				Robot *cb = g->bots [b];

				dif = pos - cb->pos;
				dis = dif.x*dif.x+dif.y*dif.y;

				float r = cb->radius + radius;
				if (dis < r*r)
					return true;
			}
		}
	}

	return false;
}

Robot *Find_Bot  (Vec2 pos, float r)
{
	int a,b;

	for (a=0;a<cbuf_w*cbuf_h;a++)
	{
		CircleGrid *g = &circles [a];
		Vec2 dif;
		float dis;

		dif = g->pos - pos;
		if (dif.x*dif.x+dif.y*dif.y < cbuf_r * cbuf_r)
		{
			for (b=0;b<g->num;b++)
			{
				Robot *cb = g->bots [b];
				if (cb->owner->is_user)
					continue;

				dif = pos - cb->pos;
				dis = dif.x*dif.x+dif.y*dif.y;

				float r = cb->radius + r;
				if (dis < r*r)
					return cb;
			}
		}
	}

	return 0;

}


void Free_CollisionMem ()
{
	SAFE_DELETE_ARRAY(circles);
}



struct Laser
{
	Laser () { free=true; }
	bool free;
	int ttl;
	Building *bld;
	Robot *bot;
	Robot *abot;
};

#define MLASER 200
Laser lasers[MLASER];

int FindFreeLaser ()
{
	int a;
	for(a=0;a<MLASER;a++)
		if (lasers[a].free) return a;
	return -1;
}

void Add_Laser(Building *b, Robot *bot)
{
	int i =FindFreeLaser();

	if (i>=0)
	{
		lasers[i].bld = b;
		lasers[i].bot = bot;
		lasers[i].free = false;
		lasers[i].ttl = 200;
	}
}

void Add_Laser(Robot *b, Robot *bot)
{
	int i =FindFreeLaser();

	if (i>=0)
	{
		lasers[i].bld = 0;
		lasers[i].abot = b;
		lasers[i].bot = bot;
		lasers[i].free = false;
		lasers[i].ttl = 200;
	}
}

void Draw_Lasers(BBox &cambox)
{
	int a;

	for(a=0;a<MLASER;a++)
	{
		Vec2 t;
		if(lasers[a].free)
			continue;
		if (lasers[a].bld)
			t = lasers[a].bld->world_pos - cambox.min;
		else
			t = lasers[a].abot->pos - cambox.min;
		Vec2 b = lasers[a].bot->pos - cambox.min;
		line (buffer, t.x,t.y, b.x,b.y, makecol (255,255,0));
	}
}

void Handle_Lasers ()
{
	int a;

	for(a=0;a<MLASER;a++)
	{
		if (!lasers[a].free)
		{
			lasers[a].ttl -= game_step;

			if (lasers[a].bld) {
				lasers[a].bld->damage += game_step;
				if (lasers[a].bld->damage > 1500)
				{
					Destroy_Building (lasers[a].bld);
					lasers[a].free=true;
					continue;
				}
			} else
			{
				lasers[a].abot->damage += game_step;
				if (lasers[a].abot->damage > 750)
				{
					Destroy_Robot (lasers[a].abot);
					lasers[a].free=true;
					continue;
				}
			}

			if (lasers[a].ttl < 0)
				lasers[a].free=true;
		}
	}
}




